/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mt_motor.c
 *
 * Purpose: Tcl interface to generic MX motor functions.
 */

#include <stdio.h>

#include <tcl.h>
#include <itcl.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_rec.h"
#include "mx_motor.h"
#include "mx_handl.h"
#include "mx_tcl.h"
#include "mx_tclpr.h"

MX_EXPORT int
Mx_motor_is_busy_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int busy;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: motor_is_busy record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_motor_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, busy );

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_move_absolute_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_motor_move_absolute_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double destination;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: motor_move_absolute record_handle destination" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &destination );

	if ( tcl_status != TCL_OK ) {
		if ( objv[2]->bytes == NULL ) {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Cannot interpret the object passed as a double.  "
		"The string representation of the object is not available." );
		} else {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Cannot interpret the object '%s' as a double.",
			objv[2]->bytes );
		}
		return TCL_ERROR;
	}

	mx_status = mx_motor_move_absolute( record, destination, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_get_position_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double position;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: motor_get_position record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_motor_get_position( record, &position );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, position );

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_set_position_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_motor_set_position_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double set_position;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: motor_set_position record_handle set_position" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &set_position );

	if ( tcl_status != TCL_OK ) {
		if ( objv[2]->bytes == NULL ) {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Cannot interpret the object passed as a double.  "
		"The string representation of the object is not available." );
		} else {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Cannot interpret the object '%s' as a double.",
			objv[2]->bytes );
		}
		return TCL_ERROR;
	}

	mx_status = mx_motor_set_position( record, set_position );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_soft_abort_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: motor_soft_abort record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_motor_soft_abort( record );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_immediate_abort_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	MX_RECORD *record;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: motor_immediate_abort record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_motor_immediate_abort( record );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_positive_limit_hit_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int limit_hit;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: motor_positive_limit_hit record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_motor_positive_limit_hit( record, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, limit_hit );

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_negative_limit_hit_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int limit_hit;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: motor_negative_limit_hit record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	mx_status = mx_motor_negative_limit_hit( record, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, limit_hit );

	return TCL_OK;
}

MX_EXPORT int
Mx_motor_find_home_position_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_motor_find_home_position()";

	MX_RECORD *record;
	int tcl_status;
	long direction;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: motor_find_home_position record_handle direction" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	tcl_status = Tcl_GetLongFromObj( interp, objv[2], &direction );

	if ( tcl_status != TCL_OK ) {
		if ( objv[2]->bytes == NULL ) {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"Cannot interpret the object passed as a long.  "
		"The string representation of the object is not available." );
		} else {
			(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"Cannot interpret the object '%s' as a long.",
			objv[2]->bytes );
		}
		return TCL_ERROR;
	}

	mx_status = mx_motor_find_home_position( record, (int) direction );

	if ( mx_status.code != MXE_SUCCESS )
		return TCL_ERROR;

	return TCL_OK;
}

